#include "pfilemanager.h"
#include "pcopydialog.h"
#include "pdeletedialog.h"
#include "pmkdirdialog.h"
#include "plinkfiledialog.h"
#include "pselectfilesdialog.h"
#include "pcmm_general.h"

#include <qheader.h>
#include <kdebug.h>
#include <kio/jobclasses.h>
#include <kpropertiesdialog.h>
#include <krun.h>
#include <qregexp.h>

#include <dirent.h>
#include <unistd.h>

#include "config.h"
#ifdef HAVE_FAM
#include <fam.h>
#endif


class PFileManager::PFileManagerPrivate {
public:
	PFileManagerPrivate( const KURL& current_dir )
	: new_listing( true ),
	  listing_completed( true ),
	  stat_completed( true ),
	  set_current_file( false ),
	  updating( false ),
	  have_fam( false ),
	  use_fam( false ),
	  want_fam( true ),
	  update_renamed_item( false ),
	  current_directory( current_dir ) {
	  }
	
	bool new_listing;
	bool listing_completed;
	bool stat_completed;
	bool show_hidden_files;
	bool set_current_file;
	bool updating;
	bool have_fam;
	bool use_fam;
	bool want_fam;
	bool update_renamed_item;

	PFileItemList new_files;
	PFileItemList files_to_remove;
	PFileItemList files_to_add;
	PFileItemList files_to_update;

	KURL current_directory;
	KURL new_directory;

	QString current_file;

	QString watch_ignore_oldrename;
	QString watch_ignore_newrename;
	QString last_rename;
	KFileItem *rename_file;
	KFileItem *file_properties;
};

PFileManager::PFileManager( bool left, PFileView *view_, const QString& current_dir )
: d( new PFileManagerPrivate( current_dir ) ), view( view_ ), watch( this, "watch" ),
  is_left_panel(left) {
	
	readConfig();

	connect( &watch, SIGNAL( dirty( const QString&)), this, SLOT( WatchedDirty( const QString& )));
	connect( &watch, SIGNAL( deleted( const QString&)), this, SLOT( WatchedDeleted( const QString& )));
	connect( &watch, SIGNAL( created( const QString&)), this, SLOT( WatchedCreated( const QString& )));

	connect( view, SIGNAL( itemRenamed (QListViewItem*, int, const QString&) ),
			SLOT( FileRenamed (QListViewItem*, int, const QString&) ) );

#ifdef HAVE_FAM
	// It's possible that FAM server can't be started
	FAMConnection fc;
	if (FAMOpen(&fc) ==0) {
		d->have_fam=true;
	}
	else {
		kdDebug(7001) << "Can't use FAM (fam daemon not running?)" << endl;
		d->have_fam=false;
	}
#endif
	d->use_fam = d->want_fam && d->have_fam;

	CONFIG->setGroup(is_left_panel ? LEFT_PANEL_KEY : RIGHT_PANEL_KEY);
	bool use_cache = CONFIG->readBoolEntry( "Use cache", false );
	view_cache = new PFileViewCache( view, use_cache );

        ListURL( CurrentDirectory() );
}

PFileManager::~PFileManager() {
	CONFIG->setGroup( GENERAL_KEY );
	if (CONFIG->readBoolEntry("Save configuration on exit", true))
		saveConfig();
	delete view_cache;
}

void PFileManager::Update() {
	d->updating = true;
	ListURL( CurrentDirectory() );
}

KURL& PFileManager::CurrentDirectory() {
	return d->current_directory;
}

bool PFileManager::hiddenFiles() {
    return d->show_hidden_files;
}

void PFileManager::showHiddenFiles( bool show ) {
    if ( show == d->show_hidden_files ) return;

    d->show_hidden_files = show;
    Update();
}

void PFileManager::useViewCache( bool use ) {
    view_cache->setCache( use );
}

bool PFileManager::hasViewCache() {
    return view_cache->hasCache();
}

void PFileManager::setFam( bool set ) {
    if ( d->want_fam == set ) return;
    d->want_fam = set;
    d->use_fam = d->want_fam && d->have_fam;
}	

bool PFileManager::wantFam() {
    return d->want_fam;
}

bool PFileManager::useFam() {
    return d->use_fam;
}

void PFileManager::MakeFile( KFileItem *file ) {
    if( file->name() == ".." )
        GoUp();
    else if ( file->isDir() )
        ListURL( KURL(file->url(), file->name()) );
    else if ( isTarArchive( file ) ) {
        KURL url( file->url(), file->name() );
        url.setProtocol( "tar" );
        ListURL( url );
    }
    else {
        new KRun( file->url() );
    }
}

void PFileManager::GoUp() {
    ListURL( CurrentDirectory().upURL() );
    d->current_file = CurrentDirectory().fileName();
    d->set_current_file = true;
}

void PFileManager::ListURL( const KURL& url ) {
	if ( !d->listing_completed ) {
		list_job->kill();
		d->listing_completed = true;
		d->set_current_file = false;
	}

	d->new_directory = url;
	d->new_listing = true;
	d->listing_completed = false;

        list_job = KIO::listDir( d->new_directory, false, d->show_hidden_files );

        connect( list_job, SIGNAL( entries( KIO::Job *, const KIO::UDSEntryList& ) ), this, SLOT( InsertFiles( KIO::Job *, const KIO::UDSEntryList& ) ) );
        connect( list_job, SIGNAL( redirection( KIO::Job *, const KURL& ) ), this, SLOT( Redirection( KIO::Job *, const KURL& ) ) );
        connect( list_job, SIGNAL( result( KIO::Job * ) ), this, SLOT( ResultList( KIO::Job * ) ) );
}

void PFileManager::focusFile( const KURL& file ) {
    d->current_file = file.fileName();
    d->set_current_file = true;
    KURL url ( file );
    url.setFileName(QString::null);
    ListURL( url );
}

void PFileManager::CopyFilesDlg( const KFileItemList* file_list ) {
	if ( !file_list->isEmpty() ) {
		KFileItemListIterator it(*file_list);

		KURL::List url_list;
		for ( ; it.current(); ++it )
			url_list.append( (*it)->url() );
		CopyFilesDlg( url_list );
	}
}

void PFileManager::CopyFileDlg( const KFileItem* file ) {
	CopyFileDlg( file->url() );
}

void PFileManager::MoveFilesDlg( const KFileItemList* file_list ) {
	if ( !file_list->isEmpty() ) {
		KFileItemListIterator it(*file_list);

		KURL::List url_list;
		for ( ; it.current(); ++it )
			url_list.append( (*it)->url() );
		MoveFilesDlg( url_list );
	}
}

void PFileManager::MoveFileDlg( const KFileItem* file ) {
	MoveFileDlg( file->url() );
}

void PFileManager::DeleteFilesDlg( const KFileItemList* file_list ) {
	if ( !file_list->isEmpty() ) {
		KFileItemListIterator it(*file_list);
		KURL::List url_list;
		for ( ; it.current(); ++it ) {
			url_list.append( (*it)->url() );
		}
		DeleteFilesDlg( url_list );
	}
}

void PFileManager::DeleteFileDlg( const KFileItem* file ) {
	DeleteFileDlg( file->url() );
}

void PFileManager::MakeDirectory() {
	PMkDirDialog dlg( view );
	if ( dlg.exec() ) {
		KURL new_dir = CurrentDirectory(); new_dir.addPath( dlg.newDir() );
		
		mkdir_job = KIO::mkdir( new_dir );
		connect( mkdir_job, SIGNAL( result( KIO::Job *) ), 
			this, SLOT( ResultMakeDirectory( KIO::Job *) ) );
	}
}

void PFileManager::SelectFiles() {
	PSelectFilesDialog dlg( view );
	if ( dlg.exec() && dlg.regExp().isValid() ) {
		KFileItemListIterator it( view_cache->items() );
		for ( ; it.current() ; ++it )
			if ( dlg.match( (*it)->name() ) )
				view_cache->setSelected( *it, true );
	}
}
			
void PFileManager::DeselectFiles() {
	PSelectFilesDialog dlg( view, false );
	if ( dlg.exec() && dlg.regExp().isValid() ) {
		KFileItemListIterator it( view_cache->items() );
		for ( ; it.current() ; ++it )
			if ( dlg.match( (*it)->name() ) )
				view_cache->setSelected( *it, false );
	}
}
			
void PFileManager::RenameFile( const KFileItem* file ) {
	if (file) {
		PFileViewItem *item = (PFileViewItem*)file->extraData( view );
		if (item) {
//			view->rename( item, 0 );
		    item->setRenameEnabled( view->cols[0], true );
		    item->startRename(view->cols[0]);
		}
	}
}

void PFileManager::LinkFilesDlg( const KFileItemList *file_list) {
	if ( !file_list->isEmpty() ) {
		KFileItemListIterator it(*file_list);
		KURL::List url_list;
		for ( ; it.current(); ++it ) {
			url_list.append( (*it)->url() );
		}
		LinkFilesDlg( url_list );
	}
}

void PFileManager::LinkFileDlg( const KFileItem *file) {
	LinkFileDlg( file->url() );
}

void PFileManager::FileProperties( KFileItem *file ) {
	KPropertiesDialog *dlg = new KPropertiesDialog( file, view, "propertiesdialog", true );
	if (!d->use_fam) {
		d->file_properties = file;
		connect( dlg, SIGNAL( applied() ), SLOT( filePropertiesChanged() ) );
	}
}

void PFileManager::CopyFiles( const QStringList& file_list, const KURL& dest ) {
	if ( !file_list.isEmpty() ) {
		QValueListConstIterator<QString> it;

		KURL::List url_list;
		for ( it = file_list.begin(); it != file_list.end(); ++it )
			url_list.append( KURL( *it ) );
		CopyFiles( url_list, dest );
	}
}

void PFileManager::CopyFile( const QString& file, const KURL& dest ) {
	CopyFile( KURL( file ), dest );
}

void PFileManager::MoveFiles( const QStringList& file_list, const KURL& dest ) {
	if ( !file_list.isEmpty() ) {
		QValueListConstIterator<QString> it;

		KURL::List url_list;
		for ( it = file_list.begin(); it != file_list.end(); ++it )
			url_list.append( KURL(*it) );
		MoveFiles( url_list, dest );
	}
}

void PFileManager::MoveFile( const QString& file, const KURL& dest ) {
	MoveFile( KURL( file ), dest );
}

void PFileManager::DeleteFiles( const QStringList& file_list ) {
	if ( !file_list.isEmpty() ) {
		QValueListConstIterator<QString> it;

		KURL::List url_list;
		for ( it = file_list.begin(); it != file_list.end(); ++it )
			url_list.append( KURL(*it) );
		DeleteFiles( url_list );
	}
}

void PFileManager::DeleteFile( const QString& file ) {
	DeleteFile( KURL( file ) );
}

void PFileManager::LinkFiles( const QStringList &file_list, const KURL& dest) {
	if ( !file_list.isEmpty() ) {
		QValueListConstIterator<QString> it;

		KURL::List url_list;
		for ( it = file_list.begin(); it != file_list.end(); ++it )
			url_list.append( KURL(*it) );
		LinkFiles( url_list, dest );
	}
}

void PFileManager::LinkFile( const QString &file, const KURL& dest) {
	LinkFile( KURL( file ), dest );
}

void PFileManager::saveConfig() {
    KConfig *config = CONFIG;
    config->setGroup(is_left_panel ? LEFT_PANEL_KEY : RIGHT_PANEL_KEY);
    config->writeEntry( "Show hidden files", d->show_hidden_files );
    config->writeEntry( "Use fam", d->want_fam );
    config->writeEntry( "Use cache", view_cache->hasCache() );
    config->sync();
}

void PFileManager::readConfig() {
    KConfig *config = CONFIG;
    config->setGroup(is_left_panel ? LEFT_PANEL_KEY : RIGHT_PANEL_KEY );
    d->show_hidden_files = config->readBoolEntry( "Show hidden files", false );
    d->want_fam = config->readBoolEntry( "Want fam", true );
}

void PFileManager::listAsOtherPanel() {
    ListURL( view->manager_other->CurrentDirectory() );
}

void PFileManager::ResultList( KIO::Job *job ) {
	d->listing_completed = true;
	view_cache->ClearCache();

	if (job->error()) {
		job->showErrorDialog();
		return;
	}
	
	if (d->updating) {
		d->updating = false;
		changeToNewDir();
		d->new_files.sort();
		CompareUpdated();
//		kdDebug(1) << "Zacinam updatovat cele view na URL: " << CurrentDirectory().prettyURL() << endl;
		view_cache->UpdateView( d->files_to_add, d->files_to_remove, d->files_to_update );
	}
}

void PFileManager::InsertFiles( KIO::Job *, const KIO::UDSEntryList& list) {
	static const QString &dot_caption = KGlobal::staticQString(".");
	KIO::UDSEntryListConstIterator it;
	it = list.begin();
	if (d->new_listing) {
		d->new_listing = false;
		d->new_files.clear();

		//vlozim svuj domovsky adresar (..) a odstranim .. a . z list => vlozim jej i tam,
		//kde se nelistuje
		KURL url( d->new_directory ); url.addPath( ".." );
		changeToNewDir();
                KFileItem *file;
		d->new_files.append( file = new KFileItem( KFileItem::Unknown, KFileItem::Unknown, url, true ) );
		if (!d->updating) view_cache->InsertItem( file, false, true );
		while (it != list.end() ) {
			file = new KFileItem( *it, d->new_directory, false, true );
			//vyuzivam toho, ze soubory . a .. jsou hned za sebou
			if ( file->name() == "." || file->name() == ".." ) {
				delete file;
				++it;
			}
			else {
				d->new_files.append( file );
				if (!d->updating) view_cache->InsertItem( file, d->set_current_file && d->current_file == file->name() );
				++it;
				break;
			}
		}

	}
	for ( ; it != list.end(); ++it ) {
		KFileItem *file = new KFileItem( *it, d->new_directory, false, true );
		d->new_files.append( file );
		if (!d->updating) view_cache->InsertItem( file, d->set_current_file && d->current_file == file->name() );
	}
}

void PFileManager::Redirection( KIO::Job *, const KURL& url) {
	d->new_directory = url;
}

void PFileManager::FileRenamed( QListViewItem* item, int , const QString& str ) {
	KFileItem *file = ((PFileViewItem*)item)->fileInfo();
	if (file->name() == str)
		return;

	KURL new_name = CurrentDirectory(); new_name.addPath( str );
	rename_job = KIO::rename( file->url(), new_name, false );
	connect( rename_job, SIGNAL( result( KIO::Job *) ), 
		this, SLOT( ResultRenameFile( KIO::Job *) ) );
	d->watch_ignore_oldrename = file->name();
	d->watch_ignore_newrename = str;
	d->last_rename = str;
	d->rename_file = file;
}

void PFileManager::ResultDelete( KIO::Job *job ) {
	if (job->error()) {
		job->showErrorDialog();
	}
	if (!d->use_fam)
		Update();
}

void PFileManager::ResultCopy( KIO::Job *job ) {
	if (job->error())
		job->showErrorDialog();
	if (!d->use_fam) {
		KIO::CopyJob *copy_job = (KIO::CopyJob*)job;
		if ( copy_job->destURL() == CurrentDirectory() )
			Update();
		if ( copy_job->destURL() == view->manager_other->CurrentDirectory() )
			view->manager_other->Update();
	}
}

void PFileManager::ResultMove( KIO::Job *job ) {
	if (job->error())
		job->showErrorDialog();
	if (!d->use_fam) {
		KIO::CopyJob *copy_job = (KIO::CopyJob*)job;
		if ( copy_job->destURL() == CurrentDirectory() || copy_job->srcURLs().first().directory() == CurrentDirectory().path(-1) )
			Update();
		if ( copy_job->destURL() == view->manager_other->CurrentDirectory() || copy_job->srcURLs().first().directory() == view->manager_other->CurrentDirectory().path(-1) )
			view->manager_other->Update();
	}
}

void PFileManager::CopyingDone( KIO::Job *job, const KURL &from, const KURL &, bool , bool  ) {
	if (job->error())
		job->showErrorDialog();
/*	kdDebug(1) << "I've copied " 
		   << from.prettyURL() 
		   << " to " 
		   << to.prettyURL() 
		   << endl;*/
	// jestlize "from" je primo v soucasnem adresari
	if ( /*!!test na oznaceni!!*/ from.directory() == CurrentDirectory().path(-1) ) {
		//odznacim soubor "from"
		view_cache->setSelected( from.fileName(), false );
	}
}

void PFileManager::CopyingLinkDone(  KIO::Job *job, const KURL &from, const QString& , const KURL& ) {
	if (job->error())
		job->showErrorDialog();
/*	kdDebug(1) << "I've copied " 
		   << from.prettyURL() 
		   << " to " 
		   << to.prettyURL() 
		   << endl;*/
	// jestlize "from" je primo v soucasnem adresari
	if ( /*!!test na oznaceni!!*/ from.directory() == CurrentDirectory().path(-1) ) {
		//odznacim soubor "from"
		view_cache->setSelected( from.fileName(), false );
	}
}

void PFileManager::MovingDone( KIO::Job *, const KURL &from, const KURL &, bool, bool ) {
	// jestlize "from" je primo v soucasnem adresari
	if ( !d->use_fam && (from.directory() == CurrentDirectory().path(-1) ) ) {
		//smazu soubor "from"
		view_cache->DeleteItem( from.fileName() );
	}
}

//do not use
void PFileManager::Copying( KIO::Job *, const KURL &, const KURL &to ) {
/*	kdDebug(1) << "I'm copying " 
		   << from.prettyURL() 
		   << " to " 
		   << to.prettyURL() 
		   << endl;*/
	if ( to.directory() == CurrentDirectory().path(-1) ) {
		KIO::StatJob *stat_job = KIO::stat( to, false );
		connect( stat_job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStat( KIO::Job *)));
	}
}

void PFileManager::ResultMakeDirectory( KIO::Job *job ) {
	if (job->error()) {
		job->showErrorDialog();
		return;
	}
	if ( !d->use_fam ) {
		KIO::StatJob *stat_job = KIO::stat( ((KIO::SimpleJob*)job)->url(), false );
		connect( stat_job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStatMkDir( KIO::Job *)));
	}
}

void PFileManager::ResultLinkFile( KIO::Job *job ) {
	if (job->error()) {
		job->showErrorDialog();
		return;
	}
	if ( !d->use_fam ) {
		KIO::CopyJob *copy_job = (KIO::CopyJob*)job;
		if ( copy_job->destURL() == CurrentDirectory() ) {
			KIO::StatJob *stat_job = KIO::stat( ((KIO::SimpleJob*)job)->url(), false );
			connect( stat_job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStatLinkFile( KIO::Job *)));
		}
	}
}

void PFileManager::ResultRenameFile( KIO::Job *job ) {
	QString renamed = d->last_rename;

	if (job->error()) {
		job->showErrorDialog();
		d->update_renamed_item = true;
	}

	KURL url( ((KIO::SimpleJob*)job)->url() );
	url.setFileName( renamed );
	KIO::StatJob *stat_job = KIO::stat( url, false );
	connect( stat_job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStatRenameFile( KIO::Job *)));
}

//only if you do not have fam service
void PFileManager::ResultStatMkDir( KIO::Job *job ) {
	const KIO::UDSEntry& entry = ((KIO::StatJob*)job)->statResult();
	KFileItem *file = new KFileItem( entry, ((KIO::StatJob*)job)->url() );
	view_cache->InsertItem( file, true );
}

void PFileManager::ResultStatLinkFile( KIO::Job *job ) {
	const KIO::UDSEntry& entry = ((KIO::StatJob*)job)->statResult();
	KFileItem *file = new KFileItem( entry, ((KIO::StatJob*)job)->url() );
	view_cache->InsertItem( file, true );
}

void PFileManager::ResultStatRenameFile( KIO::Job *job ) {
	const KIO::UDSEntry& entry = ((KIO::StatJob*)job)->statResult();
	KFileItem *file = new KFileItem( entry, ((KIO::StatJob*)job)->url() );
	d->rename_file->setURL( file->url() );
	PFileViewItem *item = (PFileViewItem*)d->rename_file->extraData( view );
	if ( item && d->update_renamed_item )
		item->init();
	d->update_renamed_item = false;
	delete file;
}

void PFileManager::ResultStatDirty( KIO::Job *job ) {
	d->stat_completed = true;
	ResultStat( job );
}

void PFileManager::ResultStat( KIO::Job *job ) {
	if (job->error() ) {
		if (job->error() != KIO::ERR_DOES_NOT_EXIST) job->showErrorDialog();
		return;
	}
	const KIO::UDSEntry& entry = ((KIO::StatJob*)job)->statResult();
	KFileItem *file = new KFileItem( entry, ((KIO::StatJob*)job)->url() );
	//podivej se jestli je v soucasnem adresari
	//jestli neni, tak nema smysl pokracovat
	if ( file && (file->url().directory(/* false*/) == CurrentDirectory().path(-1)) ) {
		//kdDebug(1) << "Updatuju nebo vytvarim soubor " << file->name() << endl;
		view_cache->UpdateItem( file );
	}
}

void PFileManager::WatchedDirty( const QString& path ) {
	kdDebug(1) << ">> Dirty: " << path << endl;
	if ( path == CurrentDirectory().path(-1) )
		Update();
	// updatuju soubor ==> je zapnuta sluzba FAM
	if (!d->use_fam) return;
	else if ( d->stat_completed ) {
		KURL url( path );
		if ( url.fileName()[0] == '.' && !d->show_hidden_files ) return;
		d->stat_completed = false;
		stat_job = KIO::stat( url, false );
		connect( stat_job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStatDirty( KIO::Job *)));
	}
}

void PFileManager::WatchedCreated( const QString& path ) {
	KURL url( path );
	if ( url.fileName()[0] == '.' && !d->show_hidden_files ) return;
	if ( url.fileName() == d->watch_ignore_newrename )
		d->watch_ignore_newrename = QString::null;
	else {
		// soubor nebyl prejmenovan
		// byl vytvoren soubor "path" ==> je zapnuta sluzba FAM
		if (!d->use_fam) return;
		kdDebug(1) << ">> Created: " << path << endl;

		KIO::StatJob *job = KIO::stat( url, false );
		connect( job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStat( KIO::Job *)));
	}
}

void PFileManager::WatchedDeleted( const QString& path ) {
	if ( path == CurrentDirectory().path(-1) )
		// byl smazan aktualni adresar
		ListURL( KURL("/") );
	else {
		KURL url( path );
		if ( url.fileName()[0] == '.' && !d->show_hidden_files ) return;
		if ( url.fileName() == d->watch_ignore_oldrename )
			d->watch_ignore_oldrename = QString::null;
		else {

			// soubor nebyl prejmenovan
			// byl smazan soubor "path" ==> je zapnuta sluzba FAM
			if (!d->use_fam) return;
			kdDebug(1) << ">> Deleted: " << path << endl;

			view_cache->DeleteItem( url.fileName() );
		}
	}
}

void PFileManager::filePropertiesChanged() {
	KFileItem *file = d->file_properties;
	if (file) {
		KIO::StatJob *stat_job = KIO::stat( file->url(), false );
		connect( stat_job, SIGNAL( result( KIO::Job *)), this, SLOT( ResultStat( KIO::Job *)));
	}
}

void PFileManager::CompareUpdated() {
	KFileItemListIterator it_new( d->new_files );
	KFileItemListIterator it( view_cache->items() );
	d->files_to_remove.clear();
	d->files_to_add.clear();
	d->files_to_update.clear();

	for( ; it.current(); ++it ) {
		for( ; it_new.current() && (it_new.current()->name() < it.current()->name()); ++it_new)
			d->files_to_add.append( it_new.current() );
		if (it_new.current() && (it_new.current()->name() == it.current()->name() )) {
			if ( HasToBeUpdated( *it, *it_new ) ) {
				KFileItem *file = *it_new;
				file->setExtraData( view_cache, *it );
				d->files_to_update.append( file );
			}
			++it_new;
		}
		else
			d->files_to_remove.append( it.current() );
	}
	for( ; it_new.current(); ++it_new )
		d->files_to_add.append( it_new.current() );
}

bool PFileManager::HasToBeUpdated( KFileItem *update, KFileItem *by ) {
	if ( update->size() == by->size() &&
		update->time(KIO::UDS_MODIFICATION_TIME) == by->time(KIO::UDS_MODIFICATION_TIME) &&
		update->permissions() == by->permissions() &&
		update->mode() == by->mode() &&
		update->user() == by->user() &&
		update->group() == by->group() )
		return false;
	else
		return true;
}

bool PFileManager::isTarArchive( KFileItem *file ) {
	return file->name().find( QRegExp( "(\\.tar|\\.tar\\.bz2|\\.tar\\.bz|\\.tar\\.gz)$" ) ) != -1;
}

void PFileManager::CopyFiles( const KURL::List& file_list, const KURL& dest ) {
	if ( !file_list.isEmpty() ) {
		copy_job = KIO::copy( file_list, dest );
		connect( copy_job, SIGNAL( result( KIO::Job *) ),
			this, SLOT( ResultCopy( KIO::Job *) ) );
		//due to selection
		connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
			view->manager_other, SLOT( CopyingDone(  KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );
		connect( copy_job, SIGNAL( copyingLinkDone( KIO::Job *, const KURL &, const QString&, const KURL& ) ),
			view->manager_other, SLOT( CopyingLinkDone(  KIO::Job *, const KURL &, const QString&, const KURL& ) ) );
	}
}

void PFileManager::CopyFilesDlg( const KURL::List& file_list ) {
	if ( !file_list.isEmpty() ) {
		PCopyDialog dlg( view, file_list, CurrentDirectory() );
		if ( dlg.exec() ) {
			copy_job = KIO::copy( file_list, dlg.destURL() );
			connect( copy_job, SIGNAL( result( KIO::Job *) ),
				this, SLOT( ResultCopy( KIO::Job *) ) );
			//due to selection
			connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
				view->manager_other, SLOT( CopyingDone(  KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );
			connect( copy_job, SIGNAL( copyingLinkDone( KIO::Job *, const KURL &, const QString&, const KURL& ) ),
				view->manager_other, SLOT( CopyingLinkDone(  KIO::Job *, const KURL &, const QString&, const KURL& ) ) );
		}
	}
}

void PFileManager::CopyFile( const KURL& file, const KURL& dest ) {
	KURL::List url_list;
	url_list.append( file );
	copy_job = KIO::copy( url_list, dest );
	connect( copy_job, SIGNAL( result( KIO::Job *) ),
		this, SLOT( ResultCopy( KIO::Job *) ) );
	//due to selection
	connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
		view->manager_other, SLOT( CopyingDone(  KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );
}

void PFileManager::CopyFileDlg( const KURL& file ) {
	PCopyDialog dlg( view, file, CurrentDirectory() );
	if ( dlg.exec() ) {
		KURL::List url_list;
		url_list.append( file );
		copy_job = KIO::copy( url_list, dlg.destURL() );
		connect( copy_job, SIGNAL( result( KIO::Job *) ),
			this, SLOT( ResultCopy( KIO::Job *) ) );
		//due to selection
		connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
			view->manager_other, SLOT( CopyingDone(  KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );
	}
}

void PFileManager::MoveFiles( const KURL::List& file_list, const KURL& dest ) {
	if ( !file_list.isEmpty() ) {
		copy_job = KIO::move( file_list, dest );
		connect( copy_job, SIGNAL( result( KIO::Job *) ), 
			this, SLOT( ResultMove( KIO::Job *) ) );
		/*connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
			this, SLOT( MovingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );*/
	}
}

void PFileManager::MoveFilesDlg( const KURL::List& file_list ) {
	if ( !file_list.isEmpty() ) {
		PCopyDialog dlg( view, file_list, CurrentDirectory(), false );
		if ( dlg.exec() ) {
			copy_job = KIO::move( file_list, dlg.destURL() );
			connect( copy_job, SIGNAL( result( KIO::Job *) ), 
				this, SLOT( ResultMove( KIO::Job *) ) );
			/*connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
				this, SLOT( MovingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );*/
		}
	}
}

void PFileManager::MoveFile( const KURL& file, const KURL& dest ) {
	KURL::List url_list;
	url_list.append( file );
	copy_job = KIO::move( url_list, dest );
	connect( copy_job, SIGNAL( result( KIO::Job *) ), 
		this, SLOT( ResultMove( KIO::Job *) ) );
	/*connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
		this, SLOT( MovingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );*/
}

void PFileManager::MoveFileDlg( const KURL& file ) {
	PCopyDialog dlg( view, file, CurrentDirectory(), false );
	if ( dlg.exec() ) {
		KURL::List url_list;
		url_list.append( file );
		copy_job = KIO::move( url_list, dlg.destURL() );
		connect( copy_job, SIGNAL( result( KIO::Job *) ), 
			this, SLOT( ResultMove( KIO::Job *) ) );
		/*connect( copy_job, SIGNAL( copyingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ),
			this, SLOT( MovingDone( KIO::Job *, const KURL &, const KURL &, bool, bool ) ) );*/
	}
}

void PFileManager::DeleteFiles( const KURL::List& file_list ) {
	if ( !file_list.isEmpty() ) {
		delete_job = KIO::del( file_list, true, true );
		connect( delete_job, SIGNAL( result( KIO::Job *) ), 
			this, SLOT( ResultDelete( KIO::Job *) ) );
	}
}

void PFileManager::DeleteFilesDlg( const KURL::List& file_list ) {
	if ( !file_list.isEmpty() ) {
		PDeleteDialog dlg( view, file_list );
		if ( dlg.exec() ) {
			delete_job = KIO::del( file_list, dlg.Shred(), true );
			connect( delete_job, SIGNAL( result( KIO::Job *) ), 
				this, SLOT( ResultDelete( KIO::Job *) ) );
		}
	}
}

void PFileManager::DeleteFile( const KURL& file ) {
	KURL::List url_list;
	url_list.append( file );
	
	delete_job = KIO::del( url_list, true, true );
	connect( delete_job, SIGNAL( result( KIO::Job *) ), this, SLOT( ResultDelete( KIO::Job *) ) );
}

void PFileManager::DeleteFileDlg( const KURL& file ) {
	KURL::List url_list;
	url_list.append( file );
	
	PDeleteDialog dlg( view, url_list );
	if ( dlg.exec() ) {
		delete_job = KIO::del( url_list, dlg.Shred(), true );
		connect( delete_job, SIGNAL( result( KIO::Job *) ), this, SLOT( ResultDelete( KIO::Job *) ) );
	}
}

void PFileManager::LinkFiles( const KURL::List &file_list, const KURL& dest) {
	if ( !file_list.isEmpty() ) {
		link_job = KIO::link( file_list, dest, true );
		connect( link_job, SIGNAL( result( KIO::Job * ) ),
				this, SLOT( ResultLinkFile( KIO::Job * ) ) );
		connect( link_job, SIGNAL(  copyingLinkDone ( KIO::Job *, const KURL&, const QString&, const KURL& ) ),
				view->manager_other, SLOT( CopyingLinkDone ( KIO::Job *, const KURL&, const QString&, const KURL& ) ) );
	}
}

void PFileManager::LinkFilesDlg( const KURL::List &file_list) {
	if ( !file_list.isEmpty() ) {
		PLinkFileDialog dlg( file_list.toStringList(), CurrentDirectory().prettyURL(), view );
		if (dlg.exec() ) {
			link_job = KIO::link( file_list, KURL( dlg.newLink() ), true );
			connect( link_job, SIGNAL( result( KIO::Job * ) ),
					this, SLOT( ResultLinkFile( KIO::Job * ) ) );
		}
	}
}

void PFileManager::LinkFile( const KURL &file, const KURL& dest) {
	link_job = KIO::link( file, dest, true );
	connect( link_job, SIGNAL( result( KIO::Job * ) ),
			this, SLOT( ResultLinkFile( KIO::Job * ) ) );
}

void PFileManager::LinkFileDlg( const KURL &file) {
	PLinkFileDialog dlg( file.fileName(), CurrentDirectory().prettyURL(), view );
	if (dlg.exec() ) {
		link_job = KIO::link( file, KURL( dlg.newLink() ), true );
		connect( link_job, SIGNAL( result( KIO::Job * ) ),
				this, SLOT( ResultLinkFile( KIO::Job * ) ) );
	}
}

void PFileManager::changeToNewDir() {
	//uprav adresar pro sledovani zmen
	watch.removeDir( CurrentDirectory().path(-1) );
	watch.addDir( d->new_directory.path(-1), true );
	
	CurrentDirectory() = d->new_directory;
	emit URLChanged( CurrentDirectory() );
}
