/***************************************************************************
                          pfilemanager.h  -  description
                             -------------------
    begin                : Fri Mar 22 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PFILEMANAGER_H
#define PFILEMANAGER_H

class PFileView;
class PFileViewCache;

#include "pfileview.h"
#include "pfileviewcache.h"
#include "pfileitemlist.h"

#include <kio/global.h>
#include <kio/job.h>
#include <qstring.h>
#include <kfileitem.h>

#include <qthread.h>
#include <qwaitcondition.h>
#include <kdirwatch.h>


/**
 * @short Class works with physical files.
 * 
 * Class works with physical files that are represented by @ref KFileItem, it operates
 * the viewer of files ( @ref PFileView ) and emits signals that changes state of
 * the manager.
 *
 * All operations with the viewer are through @ref PFileViewCache.
 */
class PFileManager: public QObject {
    Q_OBJECT
public:
    /**
     * Constructs an object that performs physicals operations with files
     * @param view_ a view connected with PFileManager
     */
    PFileManager( bool is_left_panel, PFileView* view_, const QString& current_dir = "file:/" );
    virtual ~PFileManager();

    /**
     * @returns current directory
     */
    KURL& CurrentDirectory();

    /**
     * @returns true if hidden files are shown
     * @see PFileManager::showHiddenFiles
     */
    bool hiddenFiles();

    /**
     * Show hidden files if @p show if true
     * @see PFileManager::hiddenFiles
     */
    void showHiddenFiles( bool show = true );

    /**
     * Use cache if @p use is true
     */
    void useViewCache( bool use = true );

    /**
     * @returns true if @ref PFileViewCache uses cache
     */
    bool hasViewCache();

    /**
     * Use fam service if @p set if true and service is available
     */
    void setFam( bool set = true );

    /**
     * @returns true if manager uses fam service
     */
    bool useFam();

    /**
     * @returns true if you want to use fam service when available
     */
    bool wantFam();

public slots:
    /**
     * Peforms an oparation with @ref KFileItem according to its type:
     * @li directory and compressed files list its content
     * @li executable files are executed
     * @li other files are opened with their application
     */
    void MakeFile( KFileItem* );

    /**
     * List parent directory
     */
    void GoUp();

    /**
     * List URL.
     */
    void ListURL( const KURL& );

    /**
     * Focus @p file in view
     */
    void focusFile( const KURL& file );

    /**
     * Relist current directory
     */
    void Update();

    /**
     * Copy one file to a directory specified by a dialog.
     */
    void CopyFilesDlg( const KFileItemList* );
    /** 
     * Like above */
    void CopyFileDlg( const KFileItem* );
    /**
     * Like above */
    void MoveFilesDlg( const KFileItemList* );
    /**
     * Like above */
    void MoveFileDlg( const KFileItem* );
    /**
     * Like above */
    void DeleteFilesDlg( const KFileItemList* );
    /**
     * Like above */
    void DeleteFileDlg( const KFileItem* );
    /**
     * Like above */
    void LinkFilesDlg( const KFileItemList* );
    /**
     * Like above
     */
    void LinkFileDlg( const KFileItem* );

    /**
     * Show @ref KPropertiesDialog
     */
    void FileProperties( KFileItem* );

    /**
     * Rename file.
     */
    void RenameFile( const KFileItem* );

    /**
     * Make new directory - opens @ref MkDirDialog
     */
    void MakeDirectory();

    /**
     * Select files - opens @ref PSelectFilesDialog
     */
    void SelectFiles();
    /**
     * Similar as above */
    void DeselectFiles();

    /**
     * Copy files to dest. It is used by drag&drop, it doesn't open a dialog
     */
    void CopyFiles( const QStringList&, const KURL& dest );
    /**
     * Similar as above */
    void CopyFile( const QString&, const KURL& dest );
    /**
     * Similar as above */
    void MoveFiles( const QStringList&, const KURL& dest );
    /**
     * Similar as above */
    void MoveFile( const QString&, const KURL& dest );
    /**
     * Similar as above */
    void DeleteFiles( const QStringList& );
    /**
     * Similar as above */
    void DeleteFile( const QString& );
    /**
     * Similar as above */
    void LinkFiles( const QStringList&, const KURL& dest );
    /**
     * Similar as above */
    void LinkFile( const QString&, const KURL& dest );

    /**
     * Save configuration of the instance
     */
    void saveConfig();

    /**
     * Read configuration of the instance
     */
    void readConfig();

    void listAsOtherPanel();

signals:
    /**
     * Current directory changed tu @p url
     */
    void URLChanged( KURL& );

private:
    /**
     * This method is called by the newly created thread...
     */
    void run();

protected slots:
    void ResultList( KIO::Job *job );
    void InsertFiles( KIO::Job *, const KIO::UDSEntryList& );
    void Redirection( KIO::Job *, const KURL& );

    void FileRenamed( QListViewItem* item, int col, const QString& str );
    void ResultDelete( KIO::Job *job );
    void ResultMakeDirectory( KIO::Job *job );
    void ResultLinkFile( KIO::Job *job );
    void ResultRenameFile( KIO::Job *job );
    void ResultCopy( KIO::Job *job );
    void ResultMove( KIO::Job *job );
    void CopyingDone( KIO::Job *, const KURL &from, const KURL &to, bool directory, bool renamed );
    void CopyingLinkDone( KIO::Job *, const KURL &from, const QString& target, const KURL &to );
    void Copying ( KIO::Job *, const KURL& from, const KURL& to );
    void MovingDone( KIO::Job *, const KURL &from, const KURL &to, bool directory, bool renamed );
    void ResultStat( KIO::Job *job );
    void ResultStatDirty( KIO::Job *job );
    void ResultStatMkDir( KIO::Job *job );
    void ResultStatLinkFile( KIO::Job *job );
    void ResultStatRenameFile( KIO::Job *job );

    void WatchedDirty( const QString& );
    void WatchedDeleted( const QString& );
    void WatchedCreated( const QString& );

    void filePropertiesChanged();

private:
    KIO::SimpleJob *mkdir_job;
    KIO::SimpleJob *rename_job;
    KIO::DeleteJob *delete_job;
    KIO::StatJob *stat_job;
    KIO::ListJob *list_job;
    KIO::CopyJob *copy_job;
    KIO::CopyJob *link_job;

    class PFileManagerPrivate;
    PFileManagerPrivate *d;

    PFileView *view;
    PFileViewCache *view_cache;

    /**
     * Porovna soubory v seznamu files a new_files,
     * soubory, ktere nejsou v new_files da do seznamu files_to_remove,
     * soubory, ktere nejsou v files vlozi do seznamu files_to_add a
     * nakonec seznam new_files strci do files vyprazdni jej
     */
    void CompareUpdated();

    /**
     * zjisti, zda je potreba updatovat "update" podle "by"
     */
    bool HasToBeUpdated( KFileItem *update, KFileItem *by );
    
    /**
     * updatuje soubor "update" podle hodnot "by" a zobrazi jej
     */
    void UpdateFile( KFileItem *update, KFileItem *by );

    /**
     * zjisti, zda je soubor tar-archiv
     */
    bool isTarArchive( KFileItem *file );

    KDirWatch watch;

    void CopyFiles( const KURL::List&, const KURL& dest );
    void CopyFilesDlg( const KURL::List& );
    void CopyFile( const KURL&, const KURL& dest );
    void CopyFileDlg( const KURL& );
    void MoveFiles( const KURL::List&, const KURL& dest );
    void MoveFilesDlg( const KURL::List& );
    void MoveFile( const KURL&, const KURL& dest );
    void MoveFileDlg( const KURL& );
    void DeleteFiles( const KURL::List& );
    void DeleteFilesDlg( const KURL::List& );
    void DeleteFile( const KURL& );
    void DeleteFileDlg( const KURL& );
    void LinkFiles( const KURL::List&, const KURL& dest );
    void LinkFilesDlg( const KURL::List& );
    void LinkFile( const KURL&, const KURL& dest );
    void LinkFileDlg( const KURL& );

    void FileProperties( const KURL& );

    void changeToNewDir();

    void listLocalDir();

    bool is_left_panel;
};

#endif
